import sys

from colors import color

from django.contrib.staticfiles.management.commands.runserver import (
    Command as StaticfilesRunserverCommand)
from django.core.servers import basehttp

from .runserver_mixin import RunserverMixin


@staticmethod
def log_local_message(message_format, *args):
    """
    Log a request so that it matches our local log format.
    """
    prefix = '{} {}'.format(color('INFO', fg=248), color('request', fg=5))
    message = message_format % args

    sys.stderr.write('{} {}\n'.format(prefix, message))

basehttp.WSGIRequestHandler.log_message = log_local_message


class Command(RunserverMixin, StaticfilesRunserverCommand):
    """
    Subclass the RunserverCommand from Staticfiles to set up our gulp
    environment.
    """

    pass



from django_extensions.management.commands.runserver_plus import (
    Command as DjangoExtensionsRunserverCommand)

from .runserver_mixin import RunserverMixin


class Command(RunserverMixin, DjangoExtensionsRunserverCommand):
    """
    Subclass the DjangoExtensionsRunserverCommand from django-extensions to set
    up our gulp environment.
    """

    pass



from __future__ import print_function

import atexit
import os
import psutil
import subprocess
import sys
import traceback

from signal import SIGTERM

from concurrent.futures import ThreadPoolExecutor

from colors import color

from django.core.management.base import CommandError
from django.conf import settings

from env_tools import load_env


class RunserverMixin(object):
    """
    Subclass the DjangoExtensionsRunserverCommand from django-extensions to set
    up our gulp environment.
    """

    pass

    def __init__(self, *args, **kwargs):
        self.cleanup_closing = False
        self.gulp_process = None

        super(RunserverMixin, self).__init__(*args, **kwargs)

    @staticmethod
    def gulp_exited_cb(future):
        if future.exception():
            print(traceback.format_exc())

            children = psutil.Process().children(recursive=True)

            for child in children:
                print('>>> Killing pid {}'.format(child.pid))

                child.send_signal(SIGTERM)

            print('>>> Exiting')

            # It would be nice to be able to raise a CommandError or use
            # sys.kill here but neither of those stop the runserver instance
            # since we're in a thread. This method is used in django as well.
            os._exit(1)

    def handle(self, *args, **options):
        try:
            env = load_env()
        except IOError:
            env = {}

        # XXX: In Django 1.8 this changes to:
        # if 'PORT' in env and not options.get('addrport'):
        #     options['addrport'] = env['PORT']

        if 'PORT' in env and not args:
            args = (env['PORT'],)

        # We're subclassing runserver, which spawns threads for its
        # autoreloader with RUN_MAIN set to true, we have to check for
        # this to avoid running gulp twice.
        if not os.getenv('RUN_MAIN', False):
            pool = ThreadPoolExecutor(max_workers=1)

            gulp_thread = pool.submit(self.start_gulp)
            gulp_thread.add_done_callback(self.gulp_exited_cb)

        return super(RunserverMixin, self).handle(*args, **options)

    def kill_gulp_process(self):
        if self.gulp_process.returncode is not None:
            return

        self.cleanup_closing = True
        self.stdout.write('>>> Closing gulp process')

        self.gulp_process.terminate()

    def start_gulp(self):
        self.stdout.write('>>> Starting gulp')

        gulp_cwd = getattr(settings, 'GULP_CWD', os.getcwd())

        gulp_command = getattr(settings,
                               'GULP_DEVELOP_COMMAND',
                               'gulp --cwd %s' % gulp_cwd)

        self.gulp_process = subprocess.Popen(
            gulp_command,
            shell=True,
            stdin=subprocess.PIPE,
            stdout=self.stdout._out,
            stderr=self.stderr._out)

        if self.gulp_process.poll() is not None:
            raise CommandError('gulp failed to start')

        self.stdout.write('>>> gulp process on pid {0}'
                          .format(self.gulp_process.pid))

        atexit.register(self.kill_gulp_process)

        self.gulp_process.wait()

        if self.gulp_process.returncode != 0 and not self.cleanup_closing:
            raise CommandError('gulp exited unexpectedly')



import os
import subprocess

from django.conf import settings
from django.contrib.staticfiles.management.commands.collectstatic import \
    Command as BaseCommand
from django.core.management.base import CommandError


class Command(BaseCommand):
    """
    A version of collectstatic that runs `gulp build --production` first.
    """

    def handle(self, *args, **options):
        if options['dry_run']:
            return

        popen_kwargs = {
            'shell': True,
            'stdin': subprocess.PIPE,
            'stdout': self.stdout._out,
            'stderr': self.stderr._out
        }

        # HACK: This command is executed without node_modules in the PATH
        # when it's executed from Heroku... Ideally we wouldn't need any
        # Heroku-specific code for this to work.
        if os.path.exists('/app/requirements.txt'):
            popen_kwargs['env'] = {
                'PATH': (os.environ['PATH'] +
                         ':/app/node_modules/.bin' +
                         ':/app/.heroku/node/bin')
            }

        gulp_cwd = getattr(settings, 'GULP_CWD', os.getcwd())
        gulp_command = getattr(
            settings, 'GULP_PRODUCTION_COMMAND', 'gulp build --cwd %s --production' % gulp_cwd)
        try:
            subprocess.check_call(gulp_command, **popen_kwargs)
        except subprocess.CalledProcessError as e:
            raise CommandError(e)

        super(Command, self).handle(*args, **options)



from setuptools import find_packages, setup

setup(
    name='django-gulp',
    author='Beau Gunderson',
    author_email='beau@beaugunderson.com',

    url='https://github.com/beaugunderson/django-gulp',

    description='Run your gulp tasks with runserver and collectstatic',
    long_description_markdown_filename='README.md',

    keywords=['django', 'gulp'],

    version='4.1.0',

    license='MIT',

    packages=find_packages(),

    install_requires=[
        'ansicolors>=1.1.8,<2.0',
        'env-tools>=2.2.0,<3.0',
        'futures>=3.1.1,<4.0;python_version<"3.2"',
        'psutil>=5.2.2,<6.0',
    ],

    setup_requires=[
        'setuptools-markdown'
    ],

    classifiers=[
        'Environment :: Web Environment',

        'Framework :: Django',

        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',

        'License :: OSI Approved :: MIT License',

        'Operating System :: POSIX',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: MacOS :: MacOS X',

        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries',
        'Topic :: Utilities',

        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
    ])
